package com.share.help.controller;

import com.share.help.Constants;
import com.share.help.entity.UserEntity;
import com.share.help.form.*;
import com.share.help.form.manager.FindActivityForm;
import com.share.help.form.manager.UpdateActivityForm;
import com.share.help.res.*;
import com.share.help.res.account.ManagerLoginRes;
import com.share.help.res.account.RegisterRes;
import com.share.help.service.ManagerService;
import com.share.help.service.UserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 管理员接口
 */
@RestController
@RequestMapping(Constants.MANAGER_INTERFACE)
public class ManagerController {

    @Autowired
    private ManagerService managerService;

    @Autowired
    private UserService userService;
    /**
     * 登录
     * @param managerLoginForm 管理员登录表单
     * @return 返回登录结果
     */
    @PostMapping(Constants.MANAGER_INTERFACE_LOGIN)
    public JSONResponse<ManagerLoginRes.Res,ManagerLoginRes> login(ManagerLoginForm managerLoginForm){
        if(!StringUtils.isAllEmpty(managerLoginForm.getManagerId(), managerLoginForm.getPassword())){
            return managerService.login(managerLoginForm);
        }else{
            return new JSONResponse<>(ManagerLoginRes.Res.form_fail, Result.FAIL);
        }
    }

    /**
     * 注册接口
     * @param managerRegisterForm 注册表单
     * @return 返回注册结果
     */
    @PostMapping(Constants.MANAGER_INTERFACE_REGISTER)
    public JSONResponse<RegisterRes,Void> register(ManagerRegisterForm managerRegisterForm){
        if(!StringUtils.isAllEmpty(managerRegisterForm.getManagerId(),managerRegisterForm.getPassword())){
            return managerService.register(managerRegisterForm);
        }else{
            return new JSONResponse<>(RegisterRes.fail,Result.FAIL);
        }
    }

    /**
     * 管理员列表
     * @param queryManager 查询表单
     * @return 返回管理员列表
     */
    @GetMapping(Constants.MANAGER_INTERFACE_LIST)
    public JSONResponse<DefaultRes, Page<List<ManagerRes>>> list(QueryManager queryManager){
            if(queryManager.getCurrentPage()!=null){
                return managerService.list(queryManager);
            }else{
                return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
            }
    }

    /**
     * 查找用户信息
     * @param findUserForm 查找表单
     * @return 返回用户列表
     */
    @GetMapping(Constants.MANAGER_INTERFACE_USER)
    public JSONResponse<DefaultRes,Page<List<UserEntity>>> find(FindUserForm findUserForm){
        if(findUserForm.getCurrentPage()!=null){
            return managerService.find(findUserForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 更新管理员
     * @param updateManager 更新管理员表单
     * @param managerId 管理员品证cookie
     * @return 返回更新结果
     */
    @PostMapping(Constants.MANAGER_INTERFACE_UPDATE)
    public JSONResponse<DefaultRes,Void> update(UpdateManager updateManager, @CookieValue(Constants.MANAGER_COOKIE) String managerId){
            if(!StringUtils.isAllEmpty(updateManager.getManagerId(),updateManager.getPassword())&&managerId!=null){
                return managerService.update(updateManager);
            }else{
                return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
            }
    }

    /**
     * 删除管理员
     * @param managerId 管理员id
     * @return 返回删除结果
     */
    @PostMapping(Constants.MANAGER_INTERFACE_DELETE)
    public JSONResponse<DefaultRes,Void> delete(@PathVariable String managerId){
        return managerService.delete(managerId);
    }

    /**
     * 更新用户信息
     * @param updateUserForm 更新用户表单
     * @return 返回更新结果
     */
    @PutMapping(Constants.MANAGER_INTERFACE_UPDATE_USER)
    public JSONResponse<DefaultRes,Void> update(UpdateUserForm updateUserForm){
        if(updateUserForm.getUserId()!=null){
            return userService.update(updateUserForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 查询活动列表
     * @param findActivityForm 活动表单
     * @return 返回活动列表
     */
    @GetMapping(Constants.MANAGER_INTERFACE_FIND_ACTIVITY)
    public JSONResponse<DefaultRes,Page<List<ManagerActivityRes>>> findActivity(FindActivityForm findActivityForm){
        if(findActivityForm.getCurrentPage()!=null){
            return managerService.findActivity(findActivityForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 更新活动信息
     * @param updateActivityForm 活动表单
     * @param activityId 活动id
     * @return 返回更新结果
     */
    @PutMapping(Constants.MANAGER_INTERFACE_UPDATE_ACTIVITY)
    public JSONResponse<DefaultRes,Void> updateActivity(UpdateActivityForm updateActivityForm,@PathVariable Long activityId){
        if(activityId!=null){
            updateActivityForm.setActivityId(activityId);
            return managerService.updateActivity(updateActivityForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    @GetMapping(Constants.MANAGER_INTERFACE_SCORE)
    public JSONResponse<DefaultRes,TimeScore> queryTimeScore(QueryScoreForm queryScoreForm){
        if(queryScoreForm.getActivityId()!=null&&StringUtils.isNotEmpty(queryScoreForm.getUserId())){
            return managerService.queryTimeScore(queryScoreForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }
}
