package com.share.help.controller;

import com.share.help.Constants;
import com.share.help.entity.custorm.FindUserRes;
import com.share.help.entity.custorm.Friend;
import com.share.help.entity.custorm.LeaveWordRes;
import com.share.help.entity.custorm.UserWithName;
import com.share.help.form.*;
import com.share.help.res.DefaultRes;
import com.share.help.res.JSONResponse;
import com.share.help.res.Page;
import com.share.help.res.Result;
import com.share.help.res.account.ModifyPwdRes;
import com.share.help.res.account.QueryRes;
import com.share.help.res.account.RegisterRes;
import com.share.help.res.account.UserLoginRes;
import com.share.help.service.LeaveWordService;
import com.share.help.service.UserService;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 用户接口
 */
@RestController
@RequestMapping(Constants.USER_INTERFACE)
public class UserController {

    @Autowired
    private UserService userService;

    @Autowired
    private LeaveWordService leaveWordService;

    /**
     * 注册
     * @param userRegisterForm 注册表单
     * @return 响应注册结果
     */
    @PutMapping(Constants.USER_INTERFACE_REGISTER)
    public JSONResponse<RegisterRes,Void> register(UserRegisterForm userRegisterForm){
        if(StringUtils.isNoneEmpty(userRegisterForm.getUserId(),userRegisterForm.getPassword(),
                userRegisterForm.getName(),userRegisterForm.getSex(),userRegisterForm.getAddress(),
                userRegisterForm.getServiceAddress(),userRegisterForm.getUserType(),
                userRegisterForm.getEmail(),userRegisterForm.getInfo())&&
                ObjectUtils.allNotNull(userRegisterForm.getMobile(),userRegisterForm.getImgFile())){
             return userService.register(userRegisterForm);
        }else {
            return new JSONResponse<>(RegisterRes.ok, Result.OK);
        }

    }

    /**
     * 登录
     * @param userLoginForm 登陆表单
     * @return 返回登陆结果
     */
    @PostMapping(Constants.USER_INTERFACE_LOGIN)
    public JSONResponse<UserLoginRes.Res, UserLoginRes> login(UserLoginForm userLoginForm){
        if(StringUtils.isNoneEmpty(userLoginForm.getUserId(),userLoginForm.getPassword())){
            return userService.login(userLoginForm);
        }else{
            return new JSONResponse<>(UserLoginRes.Res.form_error,Result.FAIL);
        }
    }

    /**
     * 查找用户
     * @param findUserForm 查找表单
     * @return 返回用户列表
     */
    @GetMapping(Constants.USER_INTERFACE_FIND)
    public JSONResponse<FindUserRes.Res,FindUserRes> find(FindUserForm findUserForm){
        if(findUserForm.getCurrentPage()!=null){
            return userService.find(findUserForm);
        }else{
            return new JSONResponse<>(FindUserRes.Res.fail,Result.FAIL);
        }
    }

    /**
     * 获取用户信息
     * @param userId 用户id
     * @return 返回用户信息
     */
    @GetMapping({Constants.USER_INTERFACE_FIND_ID,Constants.USER_INTERFACE_FIND_OWN})
    public JSONResponse<DefaultRes, QueryRes> find(@PathVariable(required = false) String targetUserId,@CookieValue(Constants.USER_COOKIE) String userId){
        if(userId!=null){
            return userService.find(StringUtils.isNotEmpty(targetUserId)?targetUserId:userId,userId);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 更新用户信息
     * @param updateUserForm 更新用户表单
     * @param userId 用户id
     * @return 返回更新结果
     */
    @PutMapping(Constants.USER_INTERFACE_UPDATE)
    public JSONResponse<DefaultRes,Void> update(UpdateUserForm updateUserForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(userId!=null){
            updateUserForm.setUserId(userId);
            return userService.update(updateUserForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 更新用户密码
     * @param modifyPwdForm 更新密码表单
     * @param userId 用户id
     * @return 返回更新结果
     */
    @PostMapping(Constants.USER_INTERFACE_PWD)
    public JSONResponse<ModifyPwdRes,Void> update(ModifyPwdForm modifyPwdForm, @CookieValue(Constants.USER_COOKIE) String userId){
        if(userId!=null){
            modifyPwdForm.setUserId(userId);
            return userService.update(modifyPwdForm);
        }else{
            return new JSONResponse<>(ModifyPwdRes.fail,Result.FAIL);
        }
    }



    /**
     * 留言列表
     * @return 返回留言列表
     */
    @GetMapping(Constants.USER_INTERFACE_FIND_LEAVE_WORD)
    public JSONResponse<DefaultRes, Page<List<LeaveWordRes>>> queryLeaveWord(FindLeaveWordForm findLeaveWordForm, @CookieValue(Constants.USER_COOKIE) String userId){
        if(ObjectUtils.allNotNull(findLeaveWordForm.getCurrentPage(),userId)) {
            findLeaveWordForm.setUserId(userId);
            return leaveWordService.query(findLeaveWordForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }

    }

    /**
     *发送好友邀请
     */
    @PostMapping(Constants.USER_INTERFACE_SEND_FRIEND)
    public JSONResponse<DefaultRes,Void> sendFriend(SendFriendForm sendFriendForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(ObjectUtils.allNotNull(sendFriendForm.getTargetUserId(),userId)){
            sendFriendForm.setSourceUserId(userId);
            return leaveWordService.sendFriend(sendFriendForm);
        }else {
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 拒绝好友邀请
     * @param id 留言id
     * @return 返回拒绝结果
     */
    @PostMapping(Constants.USER_INTERFACE_REFUSE_FRIEND)
    public JSONResponse<DefaultRes,Void> refuseFriend(@PathVariable Long id){
        if(id!=null){
            return leaveWordService.refuseFriend(id);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 同意好友邀请
     * @param agreeFriendForm 邀请表单
     * @param userId 邀请用户
     * @return 返回同意结果
     */
    @PostMapping(Constants.USER_INTERFACE_AGREE_FRIEND)
    public JSONResponse<DefaultRes,Void> agreeFriend(AgreeFriendForm agreeFriendForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(ObjectUtils.allNotNull(agreeFriendForm.getSourceUserId(),userId,agreeFriendForm.getLeaveWordId())){
            agreeFriendForm.setTargetUserId(userId);
            return leaveWordService.agreeFriend(agreeFriendForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 检索好友
     * @param searchFrom 检索好友表单
     * @return 返回检索结果
     */
    @GetMapping(Constants.USER_INTERFACE_FRIEND_LIST)
    public JSONResponse<DefaultRes,Page<List<Friend>>> searchFriend(SearchFrom searchFrom,@CookieValue(Constants.USER_COOKIE) String userId){
        if(ObjectUtils.isNotEmpty(searchFrom.getCurrentPage())&&StringUtils.isNotEmpty(userId)){
            searchFrom.setUserId(userId);
            return userService.searchFriend(searchFrom);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }



    /**
     * 查询用户信息
     * @param queryUserForm 搜索条件
     * @return 返回查询结果
     */
    @GetMapping(Constants.USER_INTERFACE_FIND_NAME)
    public JSONResponse<DefaultRes, Page<List<UserWithName>>> queryUser(QueryUserForm queryUserForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(userId!=null){
            queryUserForm.setUserId(userId);
            return userService.queryUser(queryUserForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail, Result.FAIL);
        }
    }

    /**
     * 刷新用户的聊天状态
     * @param chatForm 刷新表单
     * @param userId 用户id
     * @return 返回刷新结果
     */
    @PostMapping(Constants.USER_INTERFACE_CHAT_STATUS)
    public JSONResponse<DefaultRes,Void> changeChat(ChatForm chatForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(ObjectUtils.allNotNull(chatForm.getChatStatus(),userId)){
            chatForm.setUserId(userId);
            return userService.changeChat(chatForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }
}
