package com.share.help.dao;

import com.share.help.entity.ActivityEntity;
import com.share.help.entity.custorm.ActivityDetail;
import com.share.help.entity.custorm.ActivitySimple;
import com.share.help.entity.custorm.FindActivityRes;
import com.share.help.entity.custorm.SeekHelpSum;
import com.share.help.form.manager.ActiveUserRes;
import com.share.help.res.ActivitySendUser;
import com.share.help.res.ManagerActivityRes;
import com.share.help.res.Page;
import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Options;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 活动数据接口
 */
public interface ActivityMapper {

    /**
     * 保存活动信息
     * @param  activityEntity 活动实体
     * @return 返回保存结果
     */
    @Insert("insert into activity(title,content,seek_help_time,activity_start_time,activity_end_time,activity_img,service_address,user_id)" +
            "value(#{title},#{content},#{seekHelpTime},#{activityStartTime},#{activityEndTime},#{activityImg},#{serviceAddress},#{userId})")
    @Options(useGeneratedKeys=true,keyColumn="activity_id",keyProperty = "activityId")
    boolean insert(ActivityEntity activityEntity);

    /**
     * 根据活动id删除活动信息
     * @param activityId 活动id
     * @return 返回删除结果
     */
    @Delete("delete from activity where activity_id=#{activityId}")
    boolean delete(@Param("activityId") Long activityId);

    /**
     * 根据活动id查找活动信息
     * @param activityId 活动id
     * @return 返回活动信息
     */
    ActivityDetail findOne(@Param("activityId") Long activityId,@Param("userId") String userId);

    /**
     * 分页查询活动
     * @param start 起始索引
     * @param count 查询个数
     * @param userId 用户id，判断该用户是否报名活动
     * @return 返回活动列表
     */
    List<ActivitySimple> getPage(@Param("start") Integer start, @Param("count") Integer count,@Param("userId") String userId,@Param("title") String title);

    /**
     * 获取活动列表分页数
     * @param pageSize 分页大小
     * @param userId 用户id，判断该用户是否报名活动
     * @return 返回分页信息
     */
    FindActivityRes count(@Param("pageSize") Integer pageSize,@Param("userId") String userId,@Param("title") String title);

    //求助信息
    List<SeekHelpSum> seekHelp(@Param("userId") String userId,@Param("start") Integer start, @Param("count") Integer count);

    //统计分页信息
    Page<List<SeekHelpSum>> seekHelpCount(@Param("pageSize") Integer pageSize,@Param("userId") String userId);

    //管理员查找活动信息
    List<ManagerActivityRes> queryActivity(@Param("start") Integer start, @Param("count") Integer count);

    //分页信息
    Page<List<ManagerActivityRes>> countActivity(@Param("pageSize") Integer pageSize);

    //修改活动信息
    boolean updateActivity(@Param("activityId") Long activityId, @Param("title") String title, @Param("content") String content,@Param("imageName") String imageName);

    //查询活跃用户
    List<ActiveUserRes> queryActiveUser(@Param("userId") String userId,@Param("serviceAddress") String serviceAddress);

    //查询活动举办者信息
    ActivitySendUser querySendUser(@Param("activityId") Long activityId);
}
