package com.share.help.dao;

import com.share.help.entity.ManagerEntity;
import com.share.help.res.ManagerRes;
import com.share.help.res.Page;
import org.apache.ibatis.annotations.*;

import java.util.List;

/**
 * 管理员数据接口
 */
public interface ManagerMapper {

    /**
     * 根据账号密码查找管理员
     * @param managerId 管理员账号
     * @return 返回管理员信息
     */
    @Select("select manager_id as managerId,password from manager where manager_id=#{managerId}")
    ManagerEntity findUser(@Param("managerId") String managerId);

    /**
     * 添加管理员
     * @param managerEntity 管理员实体类
     * @return 返回添加结果
     */
    @Insert("insert into manager(manager_id,password) value (#{managerId},#{password})")
    boolean insert(ManagerEntity managerEntity);

    /**
     * 判断管理员是否存在
     * @param managerId 管理员id
     * @return true 存在，false 不存在
     */
    @Select("select count(*) from manager where manager_id=#{managerId}")
    boolean hasUser(@Param("managerId") String managerId);

    /**
     * 统计管理员分页信息
     * @param pageSize 分页大小
     * @return 返回分页结果
     */
    @Select("select count(*) as count,ceil(count(*)/#{pageSize}) as totalPage,#{pageSize} as pageSize from manager")
    Page<List<ManagerRes>> count(@Param("pageSize") int pageSize);

    /**
     * 分页查询管理员
     * @param start 起始索引
     * @param pageSize 查询记录数
     * @return 返回管理员集合
     */
    @Select("select manager_id as managerId from manager limit #{start},#{pageSize}")
    List<ManagerRes> list(@Param("start") int start,@Param("pageSize") int pageSize);

    /**
     * 更新管理员
     * @param managerEntity 管理员实体数据
     * @return 返回更新结果
     */
    @Update("update manager set password=#{password} where manager_id=#{managerId}")
    boolean update(ManagerEntity managerEntity);

    @Delete("delete from manager where manager_id=#{managerId}")
    boolean delete(@Param("managerId") String managerId);
}
