package com.share.help.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ResourceUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.UUID;

@Service
public  class ImageService {
    private enum ImageType{
        jpg,
        jpeg,
        png
    }

    private Logger logger = LoggerFactory.getLogger(ImageService.class);

    @Value("${spring.resources.static-locations}")
    private String imgPath;
    /**
     * 保存头像文件到本地
     * @param file 头像文件
     * @return 返回头像文件名
     */
    protected String saveImg(MultipartFile file){
        String fileName;
        //判断图片类型
        if(file.getContentType()==null){
            logger.error("获取不到图片的格式");
            return null;
        }else if(file.getContentType().contains(ImageType.jpg.name())){
            fileName= UUID.randomUUID()+"."+ImageType.jpg.name();
        }else if(file.getContentType().contains(ImageType.jpeg.name())){
            fileName= UUID.randomUUID()+"."+ImageType.jpeg.name();
        }else if (file.getContentType().contains(ImageType.png.name())) {
            fileName= UUID.randomUUID()+"."+ImageType.png.name();
        } else {
            logger.error("图片文件格式不正确");
            return null;
        }

        File imgDir;
        File imgFile;
        try {
            imgDir = new File(ResourceUtils.getURL("classpath:").getPath() + imgPath.replace("classpath:/", ""));
            if(!imgDir.exists()&&imgDir.mkdirs()){
                logger.info("成功初始化上传图片目录："+imgDir.getAbsolutePath());
            }
            imgFile=new File(imgDir,fileName);
            logger.debug("图片保存到"+imgFile.getAbsolutePath());
            file.transferTo(imgFile);
            return fileName;
        } catch (IOException e) {
            e.printStackTrace();
            logger.error("保存图片文件异常信息："+e);
            return null;
        }
    }
}
