package com.share.help.service;

import com.share.help.dao.ActivityMapper;
import com.share.help.dao.FriendMapper;
import com.share.help.dao.LeaveWordMapper;
import com.share.help.entity.FriendEntity;
import com.share.help.entity.LeaveWordEntity;
import com.share.help.entity.custorm.ActivityDetail;
import com.share.help.entity.custorm.LeaveWordRes;
import com.share.help.form.*;
import com.share.help.res.DefaultRes;
import com.share.help.res.JSONResponse;
import com.share.help.res.Page;
import com.share.help.res.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.List;

@Service
public class LeaveWordService {

    @Value("${page-size.leave-word}")
    private int pageSize;

    @Autowired
    private LeaveWordMapper leaveWordMapper;

    @Autowired
    private ActivityMapper activityMapper;

    @Autowired
    private FriendMapper friendMapper;

    private Logger logger= LoggerFactory.getLogger(LeaveWordService.class);

    /**
     * 查询留言
     * @param findLeaveWordForm 查询条件
     * @return 返回留言列表
     */
    public JSONResponse<DefaultRes, Page<List<LeaveWordRes>>> query(FindLeaveWordForm findLeaveWordForm){
        Page<List<LeaveWordRes>> listPage=leaveWordMapper.count(pageSize,findLeaveWordForm.getUserId());
        listPage.setCurrentPage(findLeaveWordForm.getCurrentPage());
        List<LeaveWordRes> leaveWordRes=leaveWordMapper.select((findLeaveWordForm.getCurrentPage()-1)*pageSize,pageSize,findLeaveWordForm.getUserId());
        for(LeaveWordRes wordRes:leaveWordRes){
            if(LeaveWordType.friend!=wordRes.getType()&&wordRes.getActivityId()!=null){
                ActivityDetail activityEntity=activityMapper.findOne(wordRes.getActivityId(),findLeaveWordForm.getUserId());
                wordRes.setTitle(activityEntity.getTitle());
                wordRes.setContent(activityEntity.getContent());
            }
        }
        listPage.setBody(leaveWordRes);
        return new JSONResponse<DefaultRes,Page<List<LeaveWordRes>>>(DefaultRes.ok, Result.OK).setBody(listPage);
    }

    /**
     * 发送好友邀请
     * @param sendFriendForm 邀请表单
     * @return 返回发送结果
     */
    public JSONResponse<DefaultRes,Void> sendFriend(SendFriendForm sendFriendForm){
        LeaveWordEntity leaveWordEntity=new LeaveWordEntity();
        leaveWordEntity.setCreateTime(new Timestamp(System.currentTimeMillis()));
        leaveWordEntity.setSourceUserId(sendFriendForm.getSourceUserId());
        leaveWordEntity.setTargetUserId(sendFriendForm.getTargetUserId());
        leaveWordEntity.setType(NotActivityStatus.friend.name());
        if(leaveWordMapper.insert(leaveWordEntity)){
            return new JSONResponse<>(DefaultRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 拒绝好友邀请
     * @param id 留言id
     * @return 返回拒绝结果
     */
    public JSONResponse<DefaultRes,Void> refuseFriend(Long id){
        if(leaveWordMapper.deleteOne(id)){
            return new JSONResponse<>(DefaultRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 同意好友邀请
     * @param agreeFriendForm 邀请表单
     * @return 返回邀请结果
     */
    public JSONResponse<DefaultRes, Void> agreeFriend(AgreeFriendForm agreeFriendForm) {
        if(!leaveWordMapper.deleteOne(agreeFriendForm.getLeaveWordId())){
            logger.error("删除留言失败");
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }

        FriendEntity friendEntity=new FriendEntity();
        friendEntity.setSourceUserId(agreeFriendForm.getSourceUserId());
        friendEntity.setTargetUserId(agreeFriendForm.getTargetUserId());
        if(friendMapper.addFriend(friendEntity)){
            return new JSONResponse<>(DefaultRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }
}
