package com.share.help;

import com.share.help.dao.*;
import com.share.help.entity.ActivityHistoryEntity;
import com.share.help.entity.ManagerEntity;
import com.share.help.entity.UserEntity;
import com.share.help.entity.custorm.*;
import com.share.help.form.ActivityStatus;
import com.share.help.form.manager.ActiveUserRes;
import com.share.help.res.ActivitySendUser;
import com.share.help.res.ManagerActivityRes;
import com.share.help.res.Page;
import com.share.help.res.TimeScore;
import com.share.help.res.account.QueryRes;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mybatis.spring.annotation.MapperScan;
import org.mybatis.spring.annotation.MapperScans;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockHttpServletRequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.DigestUtils;
import org.springframework.web.context.WebApplicationContext;

import javax.servlet.http.Cookie;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@SpringBootTest
@MapperScans(@MapperScan("com.share.help.dao"))
@EnableTransactionManagement
@Transactional
class HelpApplicationTests {

	@Autowired
	private WebApplicationContext context;

	protected MockMvc mvc;

	@Autowired
	private UserMapper userMapper;

	@Autowired
	private ActivityMapper activityMapper;

	@Autowired
	private ActivityHistoryMapper activityHistoryMapper;

	@Autowired
	private LeaveWordMapper leaveWordMapper;

	@Autowired
	private FriendMapper friendMapper;

	@Autowired
	private ManagerMapper managerMapper;

	private Logger logger = LoggerFactory.getLogger(HelpApplicationTests.class);

	@BeforeEach
	public void setUp() {
		mvc = MockMvcBuilders.webAppContextSetup(context).build();
	}

	@Test
	void contextLoads() {

	}

	@Test
	public void testAnsi(){
		logger.info(UUID.randomUUID().toString().length()+"");
	}

	/**
	 *测试增删查改
	 */
	@Test
	public void testUser(){
		String userId="fff";
		String password="123456";
		UserEntity userEntity=new UserEntity();
		userEntity.setUserId(userId);
		userEntity.setPassword(password);
		userEntity.setName("张三");
		userEntity.setSex("男");
		userEntity.setHeadImg("/test.jpg");
		userEntity.setAddress("广州");
		userEntity.setServiceAddress("广州");
		userEntity.setUserType("服务发起者");
		userEntity.setMobile(15920722180L);
		userEntity.setEmail("admin@qq.com");
		userEntity.setInfo("自我介绍");
		userEntity.setTimeScore(0);
		userEntity.setChatStatus(true);
		userEntity.setAge(12);

		userMapper.insert(userEntity);

		userEntity=userMapper.findOne(userId);

		Assertions.assertEquals(password,userEntity.getPassword(),"添加成功");

		String newPassword="22222";

		userEntity.setPassword(newPassword);

		Assertions.assertTrue(userMapper.update(userEntity),"更新成功");

		userEntity=userMapper.findOne(userId);

		Assertions.assertEquals(newPassword,userEntity.getPassword());

		userEntity=userMapper.findOneWithPwd(userId,newPassword);

		Assertions.assertNotNull(userEntity);

		List<UserEntity> u=userMapper.getAll();

		Assertions.assertEquals(1,u.size());

		Assertions.assertTrue(userMapper.hasUser(userId));

		Assertions.assertTrue(userMapper.delete(userId),"删除成功");
	}

	/**
	 * 测试用户分页查询
	 */
	@Test
	public void testFindUser(){
		List<String> list=new ArrayList<String>(){{
			add("help");
		}};
		FindUserRes findUserRes=userMapper.count(10,list,null,null);

		Assertions.assertNotNull(findUserRes);

		List<UserEntity> userEntities=userMapper.getPage(0,10,list,null,null);

		Assertions.assertFalse(userEntities.isEmpty());
	}

	/**
	 * 测试用户接口
	 */
	@Test
	public void testUserRegisterInterface() throws Exception {
		MockHttpServletRequestBuilder builder = MockMvcRequestBuilders.post(Constants.USER_INTERFACE+Constants.USER_INTERFACE_REGISTER);
		mvc.perform(builder).andDo(MockMvcResultHandlers.print());
	}

	/**
	 * 测试活动分页查询
	 */
	@Test
	public void testActivity(){
	 List<ActivitySimple> activityEntityList=activityMapper.getPage(0,10,"123","");
	 Assertions.assertFalse(activityEntityList.isEmpty());
		activityEntityList=activityMapper.getPage(0,10,"123","赛亚人");
	Assertions.assertFalse(activityEntityList.isEmpty());

	}


	@Test
	public void testAddActivity(){
			ActivitySimple activityEntity=activityMapper.findOne(63L,"222");
	}

//	求助信息统计
	@Test
	public void testSeekHelpCount(){
		Page<List<SeekHelpSum>> page=activityMapper.seekHelpCount(3,"123");
		Assertions.assertEquals(10, (long) page.getCount());
		List<SeekHelpSum> seekHelpSums=activityMapper.seekHelp("123",0,3);
		Assertions.assertEquals(seekHelpSums.size(),3);
	}


	//求助信息用户
	@Test
	public void testSeekHelpUser(){
		List<UserWithName> userWithNames=activityHistoryMapper.queryUser(ActivityStatus.join.name(),47L);
		Assertions.assertFalse(userWithNames.isEmpty());
	}

	//查询用户列表
	@Test
	public void queryUser(){
		Page<List<UserWithName>> count=userMapper.countUser("6",null,"123","222", 10);
		Assertions.assertTrue(count.getCount()>0);

	 List<UserWithName> userWithNames	=userMapper.queryUser("2",null,"123","222");
	 Assertions.assertFalse(userWithNames.isEmpty());

		userWithNames	=userMapper.queryUser("2","2", "123","222");
		Assertions.assertFalse(userWithNames.isEmpty());
	}

	//留言列表
	@Test
	public void queryLeaveWord(){
		Page<List<LeaveWordRes>> listPage=leaveWordMapper.count(10,"777");
		Assertions.assertNotNull(listPage);
		List<LeaveWordRes> leaveWordRes=leaveWordMapper.select(0,3,"123");
		Assertions.assertFalse(leaveWordRes.isEmpty());
	}

	//查询我的帮助信息
	@Test
	public void queryMyhelp(){
	Page<List<HelpRes>> page=activityHistoryMapper.count("123",10);
	Assertions.assertNotNull(page);
	 List<HelpRes> list=	activityHistoryMapper.queryMyHelp(0,2,"123");
	 Assertions.assertFalse(list.isEmpty());
	}

	/**
	 * 查找好友
	 */
	@Test
	public void queryFriend(){
		Page<List<Friend>> page=friendMapper.count("123","", "2", 3);
		Assertions.assertEquals(1, (long) page.getCount());
		List<Friend> user=friendMapper.getFriend("123","", "2", 0,3);
		Assertions.assertEquals(1,user.size());
		user=friendMapper.getFriend("123","6", "广州", 0,3);
		Assertions.assertTrue(user.isEmpty());
	}

	/**
	 * 查找用户信息
	 */
	@Test
	public void queryUserInfo(){
		QueryRes queryRes= userMapper.findInfo("123","222");
		Assertions.assertNull(queryRes.getTimeScore());

		queryRes= userMapper.findInfo("123","123");
		Assertions.assertNotNull(queryRes.getTimeScore());


	}

	@Test
	public void testQueryUserCtrl() throws Exception {
		MockHttpServletRequestBuilder builder = MockMvcRequestBuilders.get(Constants.USER_INTERFACE+
				Constants.USER_INTERFACE_FIND_ID.replace("{targetUserId}","123")).cookie(new Cookie("user_cookie","777"));
		mvc.perform(builder).andDo(MockMvcResultHandlers.print());
	}

	/**
	 * 测试活动记录
	 */
	@Test
	public void testAddHistory(){
		ActivityHistoryEntity activityHistoryEntity=new ActivityHistoryEntity();
		activityHistoryEntity.setCreateTime(new Timestamp(System.currentTimeMillis()));
		activityHistoryEntity.setActivityStatus(ActivityStatus.complete.name());
		activityHistoryEntity.setActivityId(57L);
		activityHistoryEntity.setUserId("123");
	    Integer score=activityHistoryMapper.getTimeScore(57L,"123");
	    activityHistoryEntity.setTimeScore(score);

		Assertions.assertTrue(activityHistoryMapper.insert(activityHistoryEntity));
	}

	/**
	 * 测试管理员
	 */
	@Test
	@Rollback
	public void testManager(){
		ManagerEntity managerEntity=new ManagerEntity();
		managerEntity.setManagerId("123");
		managerEntity.setPassword("123");
	   	Assertions.assertTrue(managerMapper.insert(managerEntity),"账号未注册！！！");

	   	Assertions.assertTrue(managerMapper.hasUser(managerEntity.getManagerId()),"账号未注册！！！");

	   	Assertions.assertTrue(managerMapper.count(10).getCount()>0,"管理员分页查询失败");

	   	Assertions.assertFalse(managerMapper.list(0,10).isEmpty(),"管理员分页查询失败");

	   	String password= DigestUtils.md5DigestAsHex("222".getBytes());
	   	managerEntity.setPassword(password);
	   	Assertions.assertTrue(managerMapper.update(managerEntity));

	   	Assertions.assertTrue(managerMapper.delete("123"));
	}

//	测试管理员查询活动
	@Test
	public void testQueryActivity(){
		List<ManagerActivityRes> list=activityMapper.queryActivity(0,20);

		Assertions.assertFalse(list.isEmpty(),"管理员分页查询活动失败");

		Page<List<ManagerActivityRes>> page=activityMapper.countActivity(10);
		Assertions.assertTrue(page.getCount()>0,"管理员分页查询活动失败");
	}

	//更新活动信息
	@Test
	public void testUpdateActivity(){
		Assertions.assertTrue(activityMapper.updateActivity(66L,"44","", null));
	}

	//测试活跃用户
	@Test
	public void testQueryActiveUser(){
		List<ActiveUserRes> activeUserRes= activityMapper.queryActiveUser("123","广州");
		Assertions.assertEquals(2, activeUserRes.get(0).getCount());
	}

	//测试时间币
	@Test
	public void testQueryTimeScore(){
	   TimeScore timeScore= activityHistoryMapper.queryActivityHistory("123",65L);
		Assertions.assertEquals(24, (int) timeScore.getTimeScore());
	}

	//测试查询时间币
	@Test
	public void testQuerySendUser(){
		ActivitySendUser activitySendUser=activityMapper.querySendUser(63L);
		Assertions.assertEquals(20,activitySendUser.getTimeScore());
	}
}
